/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/dom-attr",
	"dijit/_Templated",
	"dijit/form/_FormValueWidget",
	"./DateTextBox",
	"./TimeTextBox",
	"dojo/text!./templates/DateTimeTextBox.html"
], function(declare, lang, domAttr, _Templated, _FormValueWidget, DateTextBox, TimeTextBox, template){
	var iForm = lang.getObject("idx.form", true); // for backward compatibility with IDX 1.2

	// module:
	//		idx/form/DateTimeTextBox
	// summary:
	//		A Combined date and time text boxes.

	/**
	 * @name idx.form.DateTimeTextBox
	 * @class Combined date and time text boxes.
	 * @augments dijit.form._FormValueWidget
	 * @augments dijit._Templated
	 */
	return iForm.DateTimePicker = declare("idx.form.DateTimeTextBox", [_FormValueWidget, _Templated], {
	/**@lends idx.form.DateTextBox*/ 
		widgetsInTemplate: true,
		templateString: template,
		
		/**
		* base class of this oneui widget
		*/
		baseClass: "idxDateTimeTextBoxWrap",
	
		/**
		 * Specifies if the value is required.
		 */
		required: false,
		readOnly: false,
		
		value: null,
		
		showPickerIcon: false,
		dateHint: "dd/mm/yyyy",
		timeHint: "hh:mm",
	
		/**
		 * Sets up event handlers.
		 */
		postCreate: function() {
			this.inherited(arguments);
			
			this.connect(this.dateTextBox, "_setValueAttr", "_updateValueAttr");
			this.connect(this.timeTextBox, "_setValueAttr", "_updateValueAttr");
			
			//a11y
			domAttr.set(this.dateTextBox.focusNode, "title", "Input Date");
			domAttr.set(this.timeTextBox.focusNode, "title", "Input Time");
		},
	
		/**
		 * Validates both date and time values.
		 * @param {Boolean} f
		 * @returns {Boolean}
		 */
		validate: function(f) {
			//A containing form will first set "_hasBeenBlurred" on this
			//widget before validating; we must pass this on to our children
			if(typeof(this._hasBeenBlurred)!== "undefined"){
				this.dateTextBox._hasBeenBlurred = (this._hasBeenBlurred || this.dateTextBox._hasBeenBlurred);
				this.timeTextBox._hasBeenBlurred = (this._hasBeenBlurred || this.timeTextBox._hasBeenBlurred);
			}
			
			//Call separately to ensure both get called - no short-circuiting
			//This is important to trigger the validation icons if needed
			var dateValid = this.dateTextBox.validate(f);
			var timeValid = this.timeTextBox.validate(f);
			
			//Return true if, and only if, both are valid
			return (dateValid && timeValid);
		},
	
		/**
		 * Returns if both date and time are valid.
		 * @param {Boolean} f
		 * @returns {Boolean}
		 */
		isValid: function(f) {
			return this.dateTextBox.isValid(f) && this.timeTextBox.isValid(f);
		},
	
		/**
		 * Retrieves the current date value.
		 * @returns {Date}
		 * @private
		 */
		_getDateAttr: function() {
			return this.dateTextBox.get("value");
		},
	
		/**
		 * Retrieves the current time value.
		 * @returns {Date}
		 * @private
		 */
		_getTimeAttr: function() {
			return this.timeTextBox.get("value");
		},
	
		/**
		 * Sets a new date value.
		 * @param {Date} value
		 * @private
		 */
		_setDateAttr: function(value) {
			this.dateTextBox.set("value", value);
		},
	
		/**
		 * Sets a new time value.
		 * @param {Date} value
		 * @private
		 */
		_setTimeAttr: function(value) {
			this.timeTextBox.set("value", value);
		},
	
		/**
		 * Retrieves a combined date and time value.
		 * @returns {Date}
		 * @private
		 */
		_getValueAttr: function() {
			return this._getCombinedValue();
		},
	
		/**
		 * Sets a new combined date and time value.
		 * @param {Date} date
		 * @private
		 */
		_setValueAttr: function(date) {
			this.value = date;
			this.dateTextBox.set("value", date);
			this.timeTextBox.set("value", date);
		},
	
		/**
		 * Specifies "disabled" attributes for date and time text boxes.
		 * @param {Boolean} disabled
		 * @private
		 */
		_setDisabledAttr: function(disabled) {
			this.inherited(arguments);	// dijit.form._FormWidget
			this.dateTextBox.set("disabled", disabled);
			this.timeTextBox.set("disabled", disabled);
		},
	
		/**
		 * Specifies "required" attributes for date and time text boxes.
		 * @param {Boolean} required
		 * @private
		 */
		_setRequiredAttr: function(required) {
			this.required = required;
			this.dateTextBox.set("required", required);
			this.timeTextBox.set("required", required);
		},
		
		/**
		 * Specifies "readOnly" attributes for date and time text boxes.
		 * @param {Boolean} readOnly
		 * @private
		 */
		_setReadOnlyAttr: function(readOnly) {
			this.readOnly = readOnly;
			this.dateTextBox.set("readOnly", readOnly);
			this.timeTextBox.set("readOnly", readOnly);
		},
	
		/**
		 * Updates the current value with a combined data and time value.
		 * @param {Date} value
		 * @private
		 */
		_updateValueAttr: function(value) {
			this.value = this._getCombinedValue();
		},
	
		/**
		 * Generates a combined data and time value.
		 * @returns {Date}
		 * @private
		 */
		_getCombinedValue: function() {
			var d = this.dateTextBox.get("value");
			var t = this.timeTextBox.get("value");
			if (!d && !t) {
				// both TextBox have no value, return null
				return null;
			}
			d = d || new Date(0);
			t = t || new Date(0);
			var dt = new Date(d.getTime() + t.getTime());
			return dt;
		},
	
		/**
		 * Resets date and time text boxes.
		 */
		reset: function(){
			this.inherited(arguments);
			this.dateTextBox.reset();
			this.timeTextBox.reset();
		},
		
		_onMouseDown: function(e) {
			// Override FormWidget#_onMouseDown
		}
	});
});
